


# Install

To install the React Auth Provider Package, you can use your preferred package manager:

```sh
```
## or

```sh
```

Please follow [this guide](https://docs.github.com/en/packages/working-with-a-github-packages-registry/working-with-the-npm-registry#installing-a-package) on how to install private packages from Github npm registry.

# Features
## AuthProvider
A context provider component that wraps your application and provides authentication-related functions and user data.

## useAuth
A hook that allows to access the authentication context. It provides functions like login, logout, and register, as well as user data and authentication status.

# Usage

**1. Wrap your top-level component `index.tsx` or `App.tsx` into the AuthProvider to get value of the context for all components inside:**

```ts

export default function App() {
  const handleAuthProviderError = (type, error) => {
    console.log(type, error);
  };

  return (
    <AuthProvider
      clientID="REACT_APP_FUSION_AUTH_CLIENT_ID"
      serverUrl="REACT_APP_AUTH_SERVER_URL"
      onError={handleAuthProviderError}
    >
     {/* Your app components */}
    </AuthProvider>
  );
}

```

Configuration Options

- clientID: Your fusionauth application's client ID.
- serverUrl: The URL of the authentication server.
- postRegisterRedirectUrl?: The URL to redirect to after successful user registration.
- postLogoutRedirectUrl?: The URL to redirect to after user logout.
- An error handler method that returns API-related errors supplied by auth package. 

**2. Use the useAuth hook to access authentication functions and user data. UseAuth hook exposes following actions and data:**

**Login**: Use this function to initiate login action. This action will open an auth interface and ask user to enter account details. Post successful login it should be able to start a user session by setting up appropriate cookies.

**logout**: Use logout function to offer users a secure method to log out from the application.

**register**: Use this function to initiate the account registration process.

**user**: Access user information using the user data.

**isUserLoading**: The isUserLoading flag communicates whether user data is currently being fetched, enabling the display of loading indicators during data retrieval.

**isAuthenticated**: Use isAuthenticated to assess the authentication status of user, managing access to authenticated features and content of the application.

**Authenticate**: Use the Authenticate component to encapsulate private routes within it.

Use the above actions, components, and the data such as user data and isUserLoading flag as required for specific use cases. Some of the use cases are listed here.

### UserProfile
```tsx

const UserProfile = () => {
  const { user, isAuthenticated, login, logout } = useAuth();

  if (isAuthenticated) {
    return (
      <div>
        <h2>Welcome, {user.name}!</h2>
        <button onClick={logout}>Logout</button>
      </div>
    );
  } else {
    return (
      <div>
        <p>Please login to view your profile.</p>
        <button onClick={login}>Login</button>
      </div>
    );
  }
};
```

### Private Routes
```tsx
  const { Authenticate } = useAuth();

  return (
    <BrowserRouter>
      <Routes>
        <Route path="/" element={<Login />} />
        <Route
          path="/profile"
          element={
            <Authenticate>
              <Profile />
            </Authenticate>
          }
        />
      </Routes>
    </BrowserRouter>
  );
```

# Setting up a virtual host
To make sure cookies works smoothly, run your website on a virtual host. Authentication server and website need to be on the same domain. Without a properly set up virtual host, auth cookies won't be saved due to how they are shared across different domains.
[Read more][https://developer.mozilla.org/en-US/docs/Web/HTTP/Cookies#define_where_cookies_are_sent]


**1. Add a custom domain to the HOSTS file - (/etc/hosts)**

- Run the command: ```sudo nano /etc/hosts```
- Add the following custom host to the hosts file:
```sh
127.0.0.1 local.veve.me
```
- Exit the editor and save the changes.

**2. Use a web server to set up proxy pass:**
- Install the Nginx server. 
For macOS, follow the instructions here: https://www.javatpoint.com/installing-nginx-on-mac

- Open the file: ```/opt/homebrew/etc/nginx/nginx.conf```
Change the line: ```listen 8080;``` to ```listen 80;```
- Add the following location configurations inside the location / block:

```perl
location / {
    proxy_pass http://127.0.0.1:3001;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection 'upgrade';
    proxy_set_header Host $host;
    proxy_cache_bypass $http_upgrade;
}
```
- Start the server
```perl
sudo nginx
```

Make sure the web server is running on port 3001, or adjust the port in the configuration accordingly.
Try to access ```local.veve.me```. It should serve the content from ```http://127.0.0.1:3001``` or ```http://localhost:3001```.

Note - Useful ngnx commands
- Start the server - ```sudo nginx```
- Stop the server - ```sudo nginx -s stop```
- Restart the server - ```sudo nginx -s stop && sudo nginx```

# Contributing
If you encounter any issues or have suggestions for improvements, please open an issue on the GitHub repository.

